/******************************************************************************/
/*                                                                            */
/* Module     : gvibExceptions.i                                              */
/*                                                                            */
/* Description: This module is the SWIG file that implements the code used    */
/*              by the C code to set an appropriate exception as outlined     */
/*              in the Python DB API Specification.                           */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Copyright (c) 2000 Going Virtual, LLC. All rights reserved.                */
/*                                                                            */
/* This library is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU Lesser General Public                 */
/* License as published by the Free Software Foundation; either               */
/* version 2.1 of the License, or (at your option) any later version.         */
/*                                                                            */
/* This library is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          */
/* Lesser General Public License for more details.                            */
/*                                                                            */
/* You should have received a copy of the GNU Lesser General Public           */
/* License along with this library; if not, write to the Free Software        */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */
/*                                                                            */
/*                                                                            */
/* Contact Information:                                                       */
/*                                                                            */
/* Going Virtual, LLC                                                         */
/* 10800 Independence Pointe Pkwy                                             */
/* Matthews, NC 28105                                                         */
/* www.goingv.com                                                             */
/*                                                                            */
/* Robert E. Tierney                                                          */
/* bob.tierney@goingv.com                                                     */
/* 1 (704) 849-0731 x109                                                      */
/*                                                                            */
/******************************************************************************/
/*
 * $Id: gvibExceptions.i,v 1.3 2000/07/11 20:04:00 btierney Exp $
 * $Revision: 1.3 $
 */

%inline
%{

/******************************************************************************/
/*                                                                            */
/* IBSetException                                                             */
/*                                                                            */
/* This function sets an exception as a prelude to returning NULL to python,  */
/* signaling an error condition.                                              */
/*                                                                            */
/******************************************************************************/
static PyObject *
IBSetException (ISC_STATUS *statusVector)
{
  long  *sv;
  long   ibCode = statusVector[1];
  long   sqlcode;
  char   msg[256];
  char   ibMsg[512];
  char   sqlMsg[512];

  PyObject *PySqlCode;
  PyObject *PyException;
  PyObject *PyValue;

  /*--------------------------------------------------------------------------*/
  /* Get the appropriate exception class to use.                              */
  /*--------------------------------------------------------------------------*/
  sqlcode     = isc_sqlcode(statusVector);
  PySqlCode   = Py_BuildValue("(l)", sqlcode);
  PyException = PyDict_GetItem(PyExcTable, PySqlCode);

  Py_DECREF(PySqlCode);

  if (!PyException)
  {
    if (sqlcode > 0)
    {
      PyException = PyExc_Warning;
    }
    else if (sqlcode < 0)
    {
      PyException = PyExc_Error;
    }
    else /* This should never happen! */
    {
      Py_INCREF(Py_None);
      return Py_None;
    }
  }

  *sqlMsg = *ibMsg = '\0';
  sv = statusVector;

  /*--------------------------------------------------------------------------*/
  /* Get the SQL Error message (There will only be one), and then loop        */
  /* through the status vector and get all the Interbase error messages.      */
  /* Each Interbase message will be separated by 'separator'.                 */
  /*--------------------------------------------------------------------------*/
  Py_BEGIN_ALLOW_THREADS

  isc_sql_interprete(sqlcode, sqlMsg, sizeof(sqlMsg));
  while (isc_interprete(msg, &sv))
  {
    if ((strlen(ibMsg) + strlen(msg)) >= sizeof(ibMsg))
      break;

    strcat(ibMsg, separator);
    strcat(ibMsg, msg);
  }

  Py_END_ALLOW_THREADS

  /*--------------------------------------------------------------------------*/
  /* Now construct a tuple of the codes and messages to be used in calling    */
  /* the exception constructor.  Then set the exception in python.  Note that */
  /* we pass &ibMsg[2] so as to skip the first separator which we don't need. */
  /*--------------------------------------------------------------------------*/
  PyValue = Py_BuildValue("(l,s,l,s)", sqlcode, sqlMsg, ibCode, &ibMsg[2]);

  PyErr_SetObject(PyException, PyValue);

  return NULL;
}


/******************************************************************************/
/*                                                                            */
/******************************************************************************/
static void
setBadParmException (PyObject *data, char *ctype, long index)
{
  char *str;
  char buffer[128];

  PyObject *PyTmp;
  PyObject *PyValue;

  PyTmp = PyObject_Str(PyObject_Type(data));
  str   = PyString_AsString(PyTmp);

  sprintf(buffer,
          "Parameter column %ld is of type %s.  You passed data of type %s.",
          index,
          ctype,
	  str);

  PyValue = Py_BuildValue("(l,s,l,s)", -1, "", -1, buffer);

  PyErr_SetObject(PyExc_DataError, PyValue);
}


/******************************************************************************/
/*                                                                            */
/******************************************************************************/
static void
setBadParmIndexException (long index, short parmcount)
{
  char *str;
  short sqlcode = -804;
  char  sqlmsg[256];
  char  ibmsg[128];

  PyObject *PyMsg;
  PyObject *PyValue;

  PyMsg = PyDict_GetItemString(msgTable, "badParmIndex");
  if (PyMsg == NULL)
  {
    sprintf(ibmsg, 
           "Index %ld exceeds the number of actual parameters %ld",
	   index,
	   parmcount);
  }
  else
  {
    str = PyString_AsString(PyMsg);
    sprintf(ibmsg, str, index, parmcount);
  }

  Py_BEGIN_ALLOW_THREADS

  isc_sql_interprete(sqlcode, sqlmsg, sizeof(sqlmsg));

  Py_END_ALLOW_THREADS

  PyValue = Py_BuildValue("(l,s,l,s)", 
                          (long) sqlcode, 
			  sqlmsg, 
			  isc_dsql_var_count_err, 
			  ibmsg);

  PyErr_SetObject(PyExc_ProgrammingError, PyValue);
}


/******************************************************************************/
/*                                                                            */
/******************************************************************************/
static void
setOverflowException (long index)
{
  char  *str;
  short sqlcode = -802;
  char  sqlmsg[256];
  char  ibmsg[128];

  PyObject *PyMsg;
  PyObject *PyValue;

  PyMsg = PyDict_GetItemString(msgTable, "overflowError");
  if (PyMsg == NULL)
  {
    sprintf(ibmsg, 
            "The data you are trying to insert into column %ld is too big.", 
	    index);
  }
  else
  {
    str = PyString_AsString(PyMsg);
    sprintf(ibmsg, str, index);
  }

  Py_BEGIN_ALLOW_THREADS

  isc_sql_interprete(sqlcode, sqlmsg, sizeof(sqlmsg));

  Py_END_ALLOW_THREADS

  PyValue = Py_BuildValue("(l,s,l,s)", 
                          (long) sqlcode, 
			  sqlmsg, 
			  isc_arith_except, 
			  ibmsg);

  PyErr_SetObject(PyExc_DataError, PyValue);
}


%}
